/*
 * Copyright (c) 2003-2017 Objective Systems, Inc.
 *
 * This software is furnished under a license and may be used and copied
 * only in accordance with the terms of such license and with the
 * inclusion of the above copyright notice. This software or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person. No title to and ownership of the software is hereby
 * transferred.
 *
 * The information in this software is subject to change without notice
 * and should not be construed as a commitment by Objective Systems, Inc.
 *
 * PROPRIETARY NOTICE
 *
 * This software is an unpublished work subject to a confidentiality agreement
 * and is protected by copyright and trade secret law.  Unauthorized copying,
 * redistribution or other use of this work is prohibited.
 *
 * The above notice of copyright on this source code product does not indicate
 * any actual or intended publication of such source code.
 *
 *****************************************************************************/
/**
 * @file OSXSDAnyTypeClass.h
 * C++ run-time base class for anyType.
 */

#ifndef _OSXSDANYTYPECLASS_H_
#define _OSXSDANYTYPECLASS_H_

#include "rtxmlsrc/OSXSDComplexType.h"
#include "rtxsrc/osSysTypes.h"
#include "rtxsrc/rtxCppAnyAttr.h"

class EXTERNXML OSXSDAnyTypeClass : public OSXSDComplexType {
protected:
   OSXMLSTRING value;
   // attributes
   OSRTObjListClass attrs; /* of OSAnyAttrClass */

public:

   OSXSDAnyTypeClass () : OSXSDComplexType()
      { value.cdata = FALSE; value.value = 0; }

   OSXSDAnyTypeClass (const OSUTF8CHAR* val) : OSXSDComplexType()
      { value.cdata = FALSE; value.value = 0; setValue (val); }

   OSXSDAnyTypeClass (const OSXSDAnyTypeClass& o) :
      OSXSDComplexType(o), attrs (o.attrs)
      { value.cdata = FALSE; value.value = 0; setValue (o.value.value); }

   void setValue (const OSUTF8CHAR* val);

   inline void setValue (const char* val)
      { setValue ((OSUTF8CHAR*)val); }

   inline void setValue (const OSXSDAnyTypeClass& o)
      { value.cdata = o.value.cdata; setValue (o.value.value); }

   inline OSUTF8CHAR* getValue ()
      { return (OSUTF8CHAR*)value.value; }

   inline OSRTObjListClass* getAttrListPtr()
      { return &attrs; }

   inline OSXMLSTRING* getXmlStrPtr ()
      { return &value; }

   void print (const char* name);

   virtual OSRTBaseType* clone () const
      { return new OSXSDAnyTypeClass(*this); }

   virtual ~OSXSDAnyTypeClass();
} ;

#endif // _OSXSDANYTYPECLASS_H_
