/*
 * Copyright (C) 2003-2017 by Objective Systems, Inc.
 *
 * This software is furnished under a license and may be used and copied
 * only in accordance with the terms of such license and with the
 * inclusion of the above copyright notice. This software or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person. No title to and ownership of the software is hereby
 * transferred.
 *
 * The information in this software is subject to change without notice
 * and should not be construed as a commitment by Objective Systems, Inc.
 *
 * PROPRIETARY NOTICE
 *
 * This software is an unpublished work subject to a confidentiality agreement
 * and is protected by copyright and trade secret law.  Unauthorized copying,
 * redistribution or other use of this work is prohibited.
 *
 * The above notice of copyright on this source code product does not indicate
 * any actual or intended publication of such source code.
 *
 *****************************************************************************/
/**
 * @file rtXmlCppEncFuncs.h
 * XML low-level C++ encode functions. These are overloaded versions of
 * C XML encode functions for use with C++.
 */
#ifndef _RTXMLCPPENCFUNCS_H_
#define _RTXMLCPPENCFUNCS_H_

#include "rtxmlsrc/osrtxml.h"
#include "rtxsrc/rtxCppDList.h"
#ifdef HAS_STL
#include "rtxsrc/rtxCppXmlSTLString.h"
#endif

class OSXSDAnyTypeClass;
class OSRTObjListClass;

/**
 * This function encodes a variable of the XSD any attribute type.
 * This is expressed as list of name/value pairs.
 *
 * @param pctxt        Pointer to context block structure.
 * @param pAnyAttrList List of name/value pair objects.
 * @return             Completion status of operation:
 *                       - 0 = success,
 *                       - negative return value is error.
 */
EXTERNXML int rtXmlCppEncAnyAttr
(OSCTXT* pctxt, OSRTObjListClass* pAnyAttrList);

/**
 * This function encodes a variable of the XSD any type.  This is considered
 * to be a fully-wrapped element of any type (for example:
 * \<myType\>myData\</myType\>)
 *
 * @param pctxt        Pointer to context block structure.
 * @param pxmlstr      Value to be encoded. This is a string containing
 *                       the fully-encoded XML text to be copied to the
 *                       output stream.
 * @param elemName     XML element name.  A name must be provided.
 *                       If an empty string is passed (""), no element tag is
 *                       added to the encoded value.
 * @param pNS          Pointer to namespace structure.
 * @return             Completion status of operation:
 *                       - 0 = success,
 *                       - negative return value is error.
 */
EXTERNXML int rtXmlEncAny
(OSCTXT* pctxt, OSRTXMLString* pxmlstr,
 const OSUTF8CHAR* elemName,  OSXMLNamespace* pNS);

#ifdef HAS_STL
/**
 * This function encodes a variable of the XSD any type.  This is considered
 * to be a fully-wrapped element of any type (for example:
 * \<myType\>myData\</myType\>)
 *
 * @param pctxt        Pointer to context block structure.
 * @param pxmlstr      Value to be encoded. This is a string containing
 *                       the fully-encoded XMLSTL text to be copied to the
 *                       output stream.
 * @param elemName     XML element name.  A name must be provided.
 *                       If an empty string is passed (""), no element tag is
 *                       added to the encoded value.
 * @param pNS          Pointer to namespace structure.
 * @return             Completion status of operation:
 *                       - 0 = success,
 *                       - negative return value is error.
 */
EXTERNXML int rtXmlEncAny
(OSCTXT* pctxt, OSXMLSTLStringClass* pxmlstr,
 const OSUTF8CHAR* elemName,  OSXMLNamespace* pNS);
#endif

/**
 * This function encodes a variable of the XSD anyType type.  This is
 * considered to be a fully-wrapped element of anyType type (for
 * example: * \<myType\>myData\</myType\>)
 *
 * @param pctxt        Pointer to context block structure.
 * @param pvalue       Value to be encoded. This is a pointer to a
 *                     OSXSDAnyTypeClass containing the fully-encoded
*                      XML text to be copied to the output stream.
 * @return             Completion status of operation:
 *                       - 0 = success,
 *                       - negative return value is error.
 */
EXTERNXML int rtXmlCppEncAnyTypeValue
(OSCTXT* pctxt, OSXSDAnyTypeClass* pvalue);

/**
 * This function encodes a start element tag value (\<elemName\>).
 * This function is for use with C++ which uses OSRTDListClass instead of
 * OSRTList (as in the C version of this function).
 *
 * @param pctxt        Pointer to context block structure.
 * @param elemName     XML element name.
 * @param pNS          XML namespace information (prefix and URI).  If the
 *                     prefix is NULL, this method will search the context's
 *                     namespace stack for a prefix to use.
 * @param pNSAttrs     List of namespace attributes to be added to element.
 * @param terminate    Add closing '>' character.
 * @return             Completion status of operation:
 *                       - 0 = success,
 *                       - negative return value is error.
 */
EXTERNXML int rtXmlCppEncStartElement (OSCTXT* pctxt,
                                    const OSUTF8CHAR* elemName,
                                    OSXMLNamespace* pNS,
                                    OSRTDListClass* pNSAttrs,
                                    OSBOOL terminate);

/**
 * This function encodes a variable of the XSD string type.
 *
 * @param pctxt        Pointer to context block structure.
 * @param pxmlstr      XML string value to be encoded.
 * @param elemName     XML element name.  A name must be provided.
 *                       If an empty string is passed (""), no element tag is
 *                       added to the encoded value.
 * @param pNS          Pointer to namespace structure.
 * @return             Completion status of operation:
 *                       - 0 = success,
 *                       - negative return value is error.
 */
EXTERNXML int rtXmlEncString
(OSCTXT* pctxt, OSRTXMLString* pxmlstr,
 const OSUTF8CHAR* elemName, OSXMLNamespace* pNS);

#ifdef HAS_STL
/**
 * This function encodes a variable of the XSD string type.
 *
 * @param pctxt        Pointer to context block structure.
 * @param pxmlstr      XMLSTL string value to be encoded.
 * @param elemName     XML element name.  A name must be provided.
 *                       If an empty string is passed (""), no element tag is
 *                       added to the encoded value.
 * @param pNS          Pointer to namespace structure.
 * @return             Completion status of operation:
 *                       - 0 = success,
 *                       - negative return value is error.
 */
EXTERNXML int rtXmlEncString
(OSCTXT* pctxt, OSXMLSTLStringClass* pxmlstlstr,
 const OSUTF8CHAR* elemName, OSXMLNamespace* pNS);
#endif
#endif
