/*
 * Copyright (c) 2003-2017 Objective Systems, Inc.
 *
 * This software is furnished under a license and may be used and copied
 * only in accordance with the terms of such license and with the
 * inclusion of the above copyright notice. This software or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person. No title to and ownership of the software is hereby
 * transferred.
 *
 * The information in this software is subject to change without notice
 * and should not be construed as a commitment by Objective Systems, Inc.
 *
 * PROPRIETARY NOTICE
 *
 * This software is an unpublished work subject to a confidentiality agreement
 * and is protected by copyright and trade secret law.  Unauthorized copying,
 * redistribution or other use of this work is prohibited.
 *
 * The above notice of copyright on this source code product does not indicate
 * any actual or intended publication of such source code.
 *
 *****************************************************************************/
/**
 * @file rtXmlCppNamespace.h
 * XML namespace handling structures and function definitions.
 */
#ifndef _RTXMLCPPNAMESPACE_H_
#define _RTXMLCPPNAMESPACE_H_

#include "rtxmlsrc/osrtxml.h"
#include "rtxsrc/OSRTBaseType.h"
#include "rtxsrc/OSRTString.h"

/**
 * This class is used to hold an XML namespace prefix to URI mapping.
 */
class OSXMLNamespaceClass : public OSXMLNamespace {
 public:
   /**
    * The default constructor sets the namespace prefix and URI values
    * to empty values.
    */
   OSXMLNamespaceClass() { prefix = 0; uri = 0; }

   /**
    * The destructor deletes the prefix and uri string variables.
    */
   ~OSXMLNamespaceClass();

   /**
    * The parameterized constructor sets the namespace prefix and URI values
    * to the given values.  A deep copy of the values is done.
    *
    * @param nsPrefix  Namespace prefix value.
    * @param nsURI     Namespace URI value.
    */
   OSXMLNamespaceClass (const OSUTF8CHAR* nsPrefix, const OSUTF8CHAR* nsURI);

   /**
    * The parameterized constructor sets the namespace prefix and URI values
    * to the given values.  A deep copy of the values is done.
    *
    * @param nsPrefix  Namespace prefix value.
    * @param nsPrefixBytes  Namespace prefix value size in bytes.
    * @param nsURI     Namespace URI value.
    * @param nsURIBytes  Namespace URI value size in bytes.
    */
   OSXMLNamespaceClass (const OSUTF8CHAR* nsPrefix, size_t nsPrefixBytes,
      const OSUTF8CHAR* nsURI, size_t nsURIBytes);

   /**
    * The copy constructor make a deep-copy of the prefix and URI values.
    */
   OSXMLNamespaceClass (const OSXMLNamespaceClass& o);

   /**
    * This method is used to get the namespace prefix value.
    */
   inline const OSUTF8CHAR* getPrefix() const { return prefix; }

   /**
    * This method is used to get the namespace URI value.
    */
   inline const OSUTF8CHAR* getURI() const { return uri; }

   /**
    * This method is used to set the namespace prefix value.
    */
   void setPrefix (const OSUTF8CHAR* nsPrefix);

   /**
    * This method is used to set the namespace URI value.
    */
   void setURI (const OSUTF8CHAR* nsURI);
} ;

#endif
