/*
 * Copyright (C) 2003-2017 by Objective Systems, Inc.
 *
 * This software is furnished under a license and may be used and copied
 * only in accordance with the terms of such license and with the
 * inclusion of the above copyright notice. This software or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person. No title to and ownership of the software is hereby
 * transferred.
 *
 * The information in this software is subject to change without notice
 * and should not be construed as a commitment by Objective Systems, Inc.
 *
 * PROPRIETARY NOTICE
 *
 * This software is an unpublished work subject to a confidentiality agreement
 * and is protected by copyright and trade secret law.  Unauthorized copying,
 * redistribution or other use of this work is prohibited.
 *
 * The above notice of copyright on this source code product does not indicate
 * any actual or intended publication of such source code.
 *
 *****************************************************************************/
/**
 * @file rtXmlpCppDecFuncs.h
 * XML low-level C++ decode functions. These are overloaded versions of
 * C XML encode functions for use with C++.
 */
#ifndef _RTXMLPCPPDECFUNCS_H_
#define _RTXMLPCPPDECFUNCS_H_

#include "rtxmlsrc/osrtxml.h"
#include "rtxmlsrc/rtXmlPull.h"
#include "rtxmlsrc/OSXSDComplexType.h"
#ifdef HAS_STL
#include "rtxsrc/rtxCppXmlSTLString.h"
EXTERNXML int rtXmlpCppDecSTLStrList (OSCTXT* pctxt, OSRTObjListClass* plist);
EXTERNXML int rtXmlpCppDecXmlStr (OSCTXT* pctxt, OSXMLSTLStringClass* outdata);
#endif
EXTERNXML int rtXmlpCppDecStrList (OSCTXT* pctxt, OSRTObjListClass* plist);
EXTERNXML int rtXmlpCppDecXmlStr (OSCTXT* pctxt, OSRTXMLString* outdata);
EXTERNXML int rtXmlpCppGetXmlnsAttrs (OSCTXT* pctxt, OSXSDComplexType* ptype);

/**
 * Class enabling parsing of an XML Schema list into strings.
 */
class EXTRTCLASS OSXMLStringListParser
{
private:
   OSXMLDataCtxt dataCtxt;
   OSCTXT* mpctxt;
   OSBOOL mbInit;             /* true if next has been called already */
   const OSUTF8CHAR* inpdata; /* pointer to dataCtxt.mData.value where next
                                 value begins (or whitespace prior to it).
                                 If null, no more data is available.
                                 Invalid until mbInit is true.
                              */

public:
   /**
    * Create a parser on the given context.  The OSXMLReader associated with
    * the context is used to read the XML value.
    *
    * @param pctxt  Holds state information and provides access to the input
    *                that is to be parsed.
    */
   OSXMLStringListParser(OSCTXT* pctxt);

   /**
    * Assign the next string from the XML schema list to value.
    * To get all of the values from the list, call this function repeatedly
    * until it returns zero or a negative value.  After that, this object can
    * no longer be used.
    *
    * @param value Receives the next string value or empty string if there isn't
    *    one.
    * @return Return value indicates the result as follows:
    *    return < 0: error
    *    return == 0: no more strings; value is assigned empty
    *    return == 1: value is assigned the next string
    */
   int next(OSRTXMLString& value);
};

#endif
